﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GrowthControl
{
    class Connection
    {
        Node node1;
        Node node2;
        Sprite graphics;
        float animationLength;

        public Node Node1
        {
            get { return node1; }
        }

        public Node Node2
        {
            get { return node2; }
        }

        public Vector2 MidPoint
        {
            get { return node1.Position + (node2.Position - node1.Position) * 0.5f; }
        }

        public Player Owner
        {
            get { return node1.Owner; }
        }

        public Connection(Node setNode1, Node setNode2)
        {
            node1 = setNode1;
            node2 = setNode2;
            graphics = new Sprite(GameServices.content);
            graphics.ChangeContent("Symbols");
            graphics.Source = new Rectangle(200, 60, 1, 5);
            graphics.Origin = new Vector2(0, 2.5f);
            animationLength = 0.05f;
            GameInterface.world.Connections.Add(this);
            node1.Connections.Add(this);
            node2.Connections.Add(this);
            if (node1.Owner == null) node1.Owner = node2.Owner;
            if (node2.Owner == null) node2.Owner = node1.Owner;
            Owner.ShowSupplies();
        }

        public void Break()
        {
            node1.Connections.Remove(this);
            node2.Connections.Remove(this);
            GameInterface.world.Connections.Remove(this);
            if (node1.Connections.Count == 0) node1.Owner = null;
            if (node2.Connections.Count == 0) node2.Owner = null;
            GameInterface.world.CheckVictory();
        }

        public void Update()
        {
            graphics.Position = node1.Position;
            graphics.Rotation = (float)Math.Atan2(node2.Position.Y - node1.Position.Y, node2.Position.X - node1.Position.X);
        }

        public void Draw()
        {
            Color color = new Color((int)(node1.Owner.PlayerColor.R * 0.5f), (int)(node1.Owner.PlayerColor.G * 0.5f), (int)(node1.Owner.PlayerColor.B * 0.5f));
            float length = Vector2.Distance(node1.Position, node2.Position);

            animationLength += (1 - animationLength) * 0.05f;
            graphics.Draw(new Vector2(length * animationLength, 1.5f), color);
        }
    }
}
