﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GrowthControl
{
    class GameWorld
    {
        List<Node> nodes;
        List<Connection> connections;
        List<Spit> spits;

        Player playerGreen;
        Player playerBrown;

        Node selected;

        int AIFailures;
        float AITimer;

        public bool multiplayer;
        public bool hardmode;
        public Player currentPlayer;
        Player victor;

        GameEnd victoryScreen;

        Sprite highlight;
        Sprite background;
        Sprite cantConnectSymbol;
        Sprite cantSupplySymbol;

        public bool cantConnect;
        public bool cantSupply;

        public bool instructionsShown;
        Instructions instructionsMenu;

        public int turns;

        public List<Node> Nodes
        {
            get { return nodes; }
        }

        public List<Connection> Connections
        {
            get { return connections; }
        }

        public List<Spit> Spits
        {
            get { return spits; }
        }

        public Player PlayerGreen
        {
            get { return playerGreen; }
        }

        public Player PlayerBrown
        {
            get { return playerBrown; }
        }

        public Node Selected
        {
            get { return selected; }
            set { selected = value; }
        }

        public void StartGame(bool setMultiplayer, bool setHardmode)
        {
            multiplayer = setMultiplayer;
            hardmode = setHardmode;

            playerGreen = new Player(new Color(0.1f, 0.7f, 0.7f));
            playerBrown = new Player(new Color(0.7f, 0.2f, 0.1f));

            nodes = new List<Node>();
            nodes.Add(playerGreen.Home);
            nodes.Add(playerBrown.Home);

            connections = new List<Connection>();
            spits = new List<Spit>();

            playerGreen.Home.Position = new Vector2(960, 1000);
            playerBrown.Home.Position = new Vector2(960, 80);

            SetStarts();
            Populate();
            currentPlayer = playerGreen;
            GameInterface.inGame = true;

            victor = null;

            playerGreen.ShowSupplies();
            playerBrown.ShowSupplies();

            highlight = new Sprite(GameServices.content);
            highlight.ChangeContent("Symbols");
            highlight.Source = new Rectangle(0, 50, 200, 200);


            background = new Sprite(GameServices.content);
            background.ChangeContent("Background");
            background.Position = new Vector2(960, 540);

            cantConnectSymbol = new Sprite(GameServices.content);
            cantConnectSymbol.ChangeContent("Symbols");
            cantConnectSymbol.Source = new Rectangle(230, 200, 25, 25);
            cantSupplySymbol = new Sprite(GameServices.content);
            cantSupplySymbol.ChangeContent("Symbols");
            cantSupplySymbol.Source = new Rectangle(200, 200, 25, 25);

            turns = 1;

            instructionsMenu = new Instructions();
        }

        void SetStarts()
        {
            PutClose(new NodeGrow(), playerGreen.Home);
            PutClose(new NodeMove(), playerGreen.Home);
            PutClose(new NodeAttack(), playerGreen.Home);

            PutClose(new NodeGrow(), playerBrown.Home);
            PutClose(new NodeMove(), playerBrown.Home);
            PutClose(new NodeAttack(), playerBrown.Home);
        }

        void PutClose(Node node, NodeHome start)
        {
            Random rnd = new Random(DateTime.Now.Millisecond);
            do
            {
                node.Position = new Vector2(start.Position.X - 150 + rnd.Next(300), start.Position.Y - 150 + rnd.Next(300));
                Connect(start, node);
            } while (InvalidPosition(node.Position) || InvalidConnection(node, start) || node.Position.Y < 30 || node.Position.Y > 1050);
            node.Owner = start.Owner;
            nodes.Add(node);
        }

        void Populate()
        {
            Random rnd = new Random(DateTime.Now.Millisecond);
            List<Node> unconnected = new List<Node>();
            for (int i = 0; i < 80; i++)
            {
                Node newNode;
                int type = rnd.Next(4);
                switch (type)
                {
                    case 0:
                        newNode = new NodeGrow();
                        break;
                    case 1:
                        newNode = new NodeMove();
                        break;
                    case 2:
                        newNode = new NodeAttack();
                        break;
                    default:
                        newNode = new NodeSupply();
                        break;
                }

                do
                {
                    newNode.Position = new Vector2(30 + rnd.Next(1860), 30 + rnd.Next(1020));
                } while (InvalidPosition(newNode.Position));
                unconnected.Add(newNode);
                nodes.Add(newNode);
            }

            if (!multiplayer)
            {
                while (unconnected.Count > 0)
                {
                    Node closestUnlinked = null;
                    //Find unconnected closest to brown home:
                    foreach (Node unconnecedNode in unconnected)
                    {
                        if (closestUnlinked == null || Vector2.Distance(unconnecedNode.Position, playerBrown.Home.Position) < Vector2.Distance(playerBrown.Home.Position, closestUnlinked.Position))
                        {
                            closestUnlinked = unconnecedNode;
                        }
                    }

                    Node closestLinked = null;
                    //Find connected closest to unconnected:
                    foreach (Node connectedNode in playerBrown.Owned)
                    {
                        if (closestLinked == null || Vector2.Distance(connectedNode.Position, closestUnlinked.Position) < Vector2.Distance(closestLinked.Position, closestUnlinked.Position))
                        {
                            closestLinked = connectedNode;
                        }
                    }

                    if (closestLinked != null)
                    {
                        Connect(closestUnlinked, closestLinked);
                        closestUnlinked.Owner = playerBrown;
                    }
                    unconnected.Remove(closestUnlinked);
                }
            }
        }

        public bool InvalidPosition(Vector2 position)
        {
            return InvalidPosition(position, null, 80);
        }

        public bool InvalidPosition(Vector2 position, Node evaluated, int tolerance)
        {
            foreach (Node node in nodes)
            {
                if (Vector2.Distance(position, node.Position) < tolerance && node != evaluated) return true;
            }
            return false;
        }

        public bool InvalidConnection(Node node1, Node node2)
        {
            foreach (Connection connection in connections)
            {
                if (node1 != connection.Node1 && node1 != connection.Node2 && node2 != connection.Node1 && node2 != connection.Node2)
                {
                    if (LineCollision(new Vector2[] { node1.Position, node2.Position }, new Vector2[] { connection.Node1.Position, connection.Node2.Position }) != Vector2.Zero)
                    {
                        return true;
                    }
                }
            }

            foreach (Node node in nodes)
            {
                if (node != node1 && node != node2)
                {
                    float t = Vector2.Dot(node.Position - node1.Position, node2.Position - node1.Position) / Vector2.Dot(node2.Position - node1.Position, node2.Position - node1.Position);
                    if (t < 0) t = 0;
                    if (t > 1) t = 1;
                    Vector2 point = node1.Position + t * (node2.Position - node1.Position);
                    if (Vector2.Distance(point, node.Position) < node.Size)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        Vector2 LineCollision(Vector2[] line1, Vector2[] line2)
        {
            float numerator1 = (line2[1].X - line2[0].X) * (line1[0].Y - line2[0].Y) - (line2[1].Y - line2[0].Y) * (line1[0].X - line2[0].X);
            float numerator2 = (line1[1].X - line1[0].X) * (line1[0].Y - line2[0].Y) - (line1[1].Y - line1[0].Y) * (line1[0].X - line2[0].X);
            float denominator = (line2[1].Y - line2[0].Y) * (line1[1].X - line1[0].X) - (line2[1].X - line2[0].X) * (line1[1].Y - line1[0].Y);

            if (Math.Abs(denominator) < 0.001f) return Vector2.Zero;

            float Ua = numerator1 / denominator;
            float Ub = numerator2 / denominator;

            if (Ua <= 1 && Ua >= 0 && Ub <= 1 && Ub >= 0)
            {
                return new Vector2(line1[0].X + Ua * (line1[1].X - line1[0].X), line1[0].Y + Ua * (line1[1].Y - line1[0].Y));
            }
            else return Vector2.Zero;
        }
        

        public void Connect(Node node1, Node node2)
        {
            new Connection(node1, node2);
        }

        public void Attack(NodeAttack attacker, Vector2 target)
        {
            Vector2 direction = target - attacker.Position;
            direction.Normalize();
            spits.Add(new Spit(attacker, direction * 600));
            attacker.Used = true;
        }

        public void CheckConnections(Player player)
        {
            List<Node> unconnected = new List<Node>(player.Owned);

            List<Node> open = new List<Node>() { player.Home };
            List<Node> closed = new List<Node>();
            while (open.Count > 0)
            {
                foreach (Connection connection in open[0].Connections)
                {
                    if (!closed.Contains(connection.Node1) && !open.Contains(connection.Node1)) open.Add(connection.Node1);
                    if (!closed.Contains(connection.Node2) && !open.Contains(connection.Node2)) open.Add(connection.Node2);
                }
                closed.Add(open[0]);
                open.RemoveAt(0);
            }

            foreach (Node recalculatedNode in closed)
            {
                unconnected.Remove(recalculatedNode);
            }
            List<Node> toDisconnect = new List<Node>();

            foreach (Node unconnectedNode in unconnected)
            {
                if (unconnectedNode.Connections.Count <= 1)
                {
                    toDisconnect.Add(unconnectedNode);
                }
            }

            foreach (Node unconnectedNode in toDisconnect)
            {
                unconnectedNode.Owner = null;
                foreach (Connection nodeConnection in unconnectedNode.Connections.ToArray())
                {
                    nodeConnection.Break();
                }
                unconnectedNode.Used = false;
            }

        }

        public void EndTurn()
        {
            if (currentPlayer == playerGreen)
            {
                CheckConnections(playerBrown);
                currentPlayer = playerBrown;
            }
            else
            {
                CheckConnections(playerGreen);
                currentPlayer = playerGreen;
                turns++;
            }

            foreach (Node node in currentPlayer.Owned)
            {
                node.Used = false;
            }
            selected = null;
            playerGreen.ShowSupplies();
            playerBrown.ShowSupplies();
        }

        public void CheckVictory()
        {
            if (playerBrown.Home.Owner == null)
            {
                victor = playerGreen;
                victoryScreen = new GameEnd(true);
            }
            else if (playerGreen.Home.Owner == null)
            {
                victor = playerBrown;
                victoryScreen = new GameEnd(false);
            }
        }

        public void Update()
        {
            cantConnect = false;
            cantSupply = false;

            cantConnectSymbol.Position = InputController.MousePosition + new Vector2(40, 40);
            cantSupplySymbol.Position = InputController.MousePosition + new Vector2(40, 40);

            if (instructionsShown)
            {
                instructionsMenu.Update();
            }
            else
            {
                if (victor == null)
                {
                    if (currentPlayer == playerGreen || multiplayer)
                    {
                        foreach (Node node in nodes)
                        {
                            node.Update();
                        }
                    }

                    foreach (Connection connection in connections)
                    {
                        connection.Update();
                    }

                    foreach (Spit spit in spits.ToArray())
                    {
                        spit.Update();
                    }

                    if (!multiplayer && currentPlayer == playerBrown)
                    {
                        if (AITimer > 30)
                        {
                            PlayAI();
                            AITimer -= 30;
                        }
                        AITimer++;
                    }
                }
                else
                {
                    victoryScreen.Update();
                }
            }

            if (InputController.KeyReleased(Microsoft.Xna.Framework.Input.Keys.Escape))
            {
                GameInterface.inGame = false;
            }
            if (InputController.KeyReleased(Microsoft.Xna.Framework.Input.Keys.F1))
            {
                instructionsShown = true;
            }
        }

        void PlayAI()
        {
            Random rnd = new Random(GameServices.time.TotalGameTime.Milliseconds);
            List<Node> unclaimed = new List<Node>();
            foreach (Node node in nodes)
            {
                if (node.Owner == null) unclaimed.Add(node);
            }
            
            List<NodeAttack> attackers = new List<NodeAttack>();
            foreach (Node node in playerBrown.Owned)
            {
                if (node is NodeAttack && !node.Used)
                {
                    attackers.Add(node as NodeAttack);
                }
            }

            if (playerBrown.Attacks > 0)
            {
                Vector2 target = new Vector2(rnd.Next(1920), rnd.Next(1080));
                if (hardmode)
                {
                    List<Connection> playersConnections = new List<Connection>();
                    foreach (Connection connection in connections)
                    {
                        if (connection.Owner == playerGreen)
                        {
                            playersConnections.Add(connection);
                        }
                    }
                    target = playersConnections[rnd.Next(playersConnections.Count)].MidPoint;
                }
                attackers[0].RotateTo(target);
                Attack(attackers[0], target);
            }

            if (playerBrown.Attacks == 0 && playerBrown.Growths > 0)
            {
                while (AIFailures < 10000)
                {
                    if (unclaimed.Count > 0 && rnd.Next(2) == 0)
                    {
                        Node target = unclaimed[rnd.Next(unclaimed.Count)];
                        if (playerBrown.Owned[rnd.Next(playerBrown.Owned.Count)].Connect(target))
                        {
                            unclaimed.Remove(target);
                            break;
                        }
                        else { AIFailures++; }

                    }
                    else
                    {
                        if (playerBrown.Owned[rnd.Next(playerBrown.Owned.Count)].Connect(playerBrown.Owned[rnd.Next(playerBrown.Owned.Count)]))
                        {
                            break;
                        }
                        else { AIFailures++; }
                    }
                }
            }

            if ((playerBrown.Growths == 0 && playerBrown.Attacks == 0) || AIFailures >= 10000)
            {
                EndTurn();
            }
        }

        public void Draw()
        {
            Matrix translation = Matrix.CreateScale(GameServices.resolution.Y / 1080); 

            GameServices.spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, SamplerState.LinearClamp, DepthStencilState.Default, RasterizerState.CullCounterClockwise, null, translation);
            background.Draw();
            if (instructionsShown)
            {
                instructionsMenu.Draw();
            }
            else
            {
                foreach (Connection connection in connections)
                {
                    connection.Draw();
                }

                foreach (Node node in nodes)
                {
                    node.Draw();
                }

                if (cantConnect) cantConnectSymbol.Draw();
                if (cantSupply) cantSupplySymbol.Draw();

                GameServices.spriteBatch.End();

                GameServices.spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.Additive, SamplerState.LinearClamp, DepthStencilState.Default, RasterizerState.CullCounterClockwise, null, translation);

                foreach (Spit spit in spits.ToArray())
                {
                    spit.Draw();
                }
                foreach (Node node in nodes)
                {
                    if (node.Owner != null)
                    {
                        highlight.Scale = 2;
                        highlight.Position = node.Position;
                        highlight.Draw(node.Owner.PlayerColor);
                    }
                }
                GameServices.spriteBatch.End();

                GameServices.spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, SamplerState.LinearClamp, DepthStencilState.Default, RasterizerState.CullCounterClockwise, null, translation);
                if (victor != null) victoryScreen.Draw();
            }
            GameServices.spriteBatch.End();

        }
    }
}
