﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GrowthControl
{
    class Instructions
    {
        Button buttonBack;
        NodeGrow nodeGrow;
        NodeMove nodeMove;
        NodeAttack nodeAttack;
        NodeSupply nodeSupply;
        NodeHome nodeHome;
        Player instructionsPlayer;


        public Instructions()
        {
            buttonBack = new Button(new Vector2(960, 1000), "Back");

            instructionsPlayer = new Player(new Color(0.1f, 0.7f, 0.7f));

            nodeGrow = new NodeGrow();
            nodeMove = new NodeMove();
            nodeAttack = new NodeAttack();
            nodeSupply = new NodeSupply();
            nodeHome = new NodeHome();

            nodeGrow.Owner = instructionsPlayer;
            nodeMove.Owner = instructionsPlayer;
            nodeAttack.Owner = instructionsPlayer;
            nodeSupply.Owner = instructionsPlayer;
            nodeHome.Owner = instructionsPlayer;

            nodeGrow.Position = new Vector2(100, 170);
            nodeMove.Position = new Vector2(100, 330);
            nodeAttack.Position = new Vector2(100, 490);
            nodeSupply.Position = new Vector2(100, 620);
            nodeHome.Position = new Vector2(100, 780);

            nodeGrow.Update();
            nodeMove.Update();
            nodeAttack.Update();
            nodeSupply.Update();
            nodeHome.Update();

            nodeSupply.freeSupplies = 2;
            nodeHome.freeSupplies = 3;
        }

        public void Update()
        {
            if (buttonBack.IsPressed())
            {
                if (GameInterface.inGame)
                {
                    GameInterface.world.instructionsShown = false;
                }
                else
                {
                    GameInterface.mainMenu.instructionsShown = false;
                }
            }
        }

        public void Draw()
        {
            string text = "Left click on any of your (cyan) nodes to select it.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(100, 70), new Color(1f, 1f, 1f));

            text = "Connection node. Provides one connection action per turn. \n" +
                "To connect nodes, select a node and then right click on friendly or neutral node.\n" + 
                "Connections can't pass through nodes or cross other connections.\n" +  
                "Each node can have as many connections as you can fit there.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(180, 140), new Color(1f, 1f, 1f));
            text = "Movement node. Provides one movement action per turn.\n" + 
                "To move a node, select it and then right click on it.\n" +
                "Once in move mode, left click to confirm, right click to cancel.\n" +  
                "Nodes can't touch other nodes and connection rules apply.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(180, 300), new Color(1f, 1f, 1f));
            text = "Attack node. Provides one attack per turn.\n" + 
                "Select it and right click on an empty spot to fire in that direction.\n" + 
                "Projectiles break any enemy connections they touch, but are blocked by nodes.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(180, 460), new Color(1f, 1f, 1f));
            text = "Supply node. Determines how many nodes you can have connected.\n" + 
                "Each provides supplies for itself and 3 other nodes.\n" + 
                "Lit up triangles represent free supply, dark ones represent supplies in use.\n" +
                "Without supplies, you can only connect to supply nodes.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(180, 590), new Color(1f, 1f, 1f));
            text = "Home node. The goal of each player is to break all connections originating from it.\n" + 
                "It provides 5 supplies (represented by edge triangles) and is immovable. To end your turn, select it and right click on it.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(180, 750), new Color(1f, 1f, 1f));

            text = "Whenever a node's action is expended, it turns dark.\n" +
                "A node with no connections is neutral (grey) and can be captured.\n" + 
                "When a cluster is disconnected from home node, all nodes with only one connection are disconnected every turn.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(100, 860), new Color(1f, 1f, 1f));

            text = "Press F1 to show this screen during game.";
            GameServices.spriteBatch.DrawString(GameInterface.smallFont, text, new Vector2(1300, 30), new Color(1f, 1f, 1f));

            buttonBack.Draw();
            nodeGrow.Draw();
            nodeMove.Draw();
            nodeAttack.Draw();
            nodeSupply.Draw();
            nodeHome.Draw();
        }
    }
}
