﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GrowthControl
{
    abstract class Node
    {
        protected Sprite graphics;
        protected Sprite outline;
        protected Sprite selection;
        protected float size = 25;
        protected Vector2 position;
        protected Vector2 oldPosition;
        protected Player owner;
        protected bool unmovable;
        protected bool used;
        protected bool preselected;
        protected List<Connection> connections;
        protected bool moveMode;
        protected Color color = new Color(0.5f, 0.5f, 0.5f);

        public Vector2 Position
        {
            get { return position; }
            set { position = value; }
        }

        public Player Owner
        {
            get { return owner; }
            set
            {
                owner = value;
                used = false;
            }
        }

        public float Size
        {
            get { return size; }
        }

        public bool Used
        {
            get { return used; }
            set { used = value; }
        }

        public bool Selected
        {
            get { return GameInterface.world.Selected == this; }
        }

        public bool MouseHover
        {
            get { return Vector2.Distance(InputController.MousePosition, Position) < size; }
        }

        public List<Connection> Connections
        {
            get { return connections; }
        }

        public void ResetConnections()
        {
            connections = new List<Connection>();
        }

        public bool Connect(Node target)
        {
            if ((target.owner == null || target.owner == owner) && this != target && !GameInterface.world.InvalidConnection(this, target) && !ConnectionExists(this, target) && (owner.Supplies > 0 || target.owner == owner|| target is NodeSupply))
            {
                new Connection(this, target);
                owner.UseGrowth();
                return true;
            }
            return false;
        }

        bool ConnectionExists(Node node1, Node node2)
        {
            foreach (Connection connection in node1.connections)
            {
                if ((connection.Node1 == node1 && connection.Node2 == node2) || (connection.Node1 == node2 && connection.Node2 == node1)) return true;
            }
            return false;
        }

        public virtual void Update()
        {
            outline.Position = position;
            graphics.Position = position;
            selection.Position = position;
            selection.Rotation += 0.01f;
            if (GameInterface.inGame && GameInterface.world.currentPlayer == owner)
            {
                if (moveMode)
                {
                    selection.Scale = 1.9f;
                    position = InputController.MousePosition;
                    if (InputController.RightRelease())
                    {
                        moveMode = false;
                        position = oldPosition;
                    }
                    else if (InputController.LeftRelease())
                    {
                        bool invalidConnection = false;
                        foreach (Connection connection in connections) {
                            if (GameInterface.world.InvalidConnection(connection.Node1, connection.Node2)) invalidConnection = true;
                        }
                        if (GameInterface.world.InvalidPosition(position, this, (int)size * 2 + 30) || invalidConnection)
                        {
                            position = oldPosition;
                        }
                        else
                        {
                            owner.UseMove();
                        }
                        moveMode = false;
                    }
                }
                else
                {
                    selection.Scale = 1f;
                    if (MouseHover && InputController.LeftPress()) preselected = true;
                    if (MouseHover && InputController.LeftRelease() && preselected) GameInterface.world.Selected = this;
                    if (preselected && InputController.LeftRelease()) preselected = false;

                    if (Selected && InputController.RightRelease())
                    {
                        if (!unmovable && MouseHover && owner.Moves > 0)
                        {
                            moveMode = true;
                            oldPosition = position;
                        }
                        else
                        {
                            foreach (Node node in GameInterface.world.Nodes)
                            {
                                if (node.MouseHover && owner.Growths > 0)
                                {
                                    Connect(node);
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            if (owner == null && MouseHover && GameInterface.world.Selected != null)
            {
                if (GameInterface.world.currentPlayer.Growths == 0)
                {
                    GameInterface.world.cantConnect = true;
                }
                else if (GameInterface.world.currentPlayer.Supplies == 0 && !(this is NodeSupply))
                {
                    GameInterface.world.cantSupply = true;
                }
            }
        }

        public virtual void Draw()
        {
            Color outlineColor = new Color(127, 127, 127);
            Color graphicsColor = new Color(127, 127, 127);

            if (owner != null)
            {
                outlineColor = owner.PlayerColor;
                graphicsColor = owner.PlayerColor;
            }

            if (MouseHover) outlineColor = new Color(outlineColor.R + 150, outlineColor.G + 150, outlineColor.B + 150);
            if (used) graphicsColor *= 0.3f;
            graphicsColor.A = 255;

            outline.Draw(outlineColor);
            graphics.Draw(graphicsColor);

            if (GameInterface.inGame && Selected)
            {
                selection.Draw(outlineColor);
            }
        }
    }
}
