﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GrowthControl
{
    class Player
    {
        NodeHome home;
        Color color;

        public int Moves
        {
            get
            {
                int count = 0;
                foreach (Node node in Owned)
                {
                    if (node is NodeMove && !node.Used)
                    {
                        count++;
                    }
                }
                return count;
            }
        }

        public int Growths
        {
            get 
            {
                int count = 0;
                foreach (Node node in Owned)
                {
                    if (node is NodeGrow && !node.Used)
                    {
                        count++;
                    }
                }
                return count;
            }
        }

        public int Attacks
        {
            get
            {
                int count = 0;
                foreach (Node node in Owned)
                {
                    if (node is NodeAttack && !node.Used)
                    {
                        count++;
                    }
                }
                return count;
            }
        }

        public int Supplies
        {
            get
            {
                int count = 0;
                List<NodeSupply> supplyNodes = new List<NodeSupply>();
                foreach (Node node in Owned)
                {
                    if (node is NodeSupply)
                    {
                        count += 3;
                        supplyNodes.Add(node as NodeSupply);
                    }
                    else if (node is NodeHome)
                    {
                        count += 5;
                    }
                    else
                    {
                        count--;
                    }
                }
                ShowSupplies(supplyNodes);
                return count;
            }
        }

        public NodeHome Home
        {
            get { return home; }
        }

        public Color PlayerColor
        {
            get { return color; }
        }

        public List<Node> Owned
        {
            get 
            {
                List<Node> owned = new List<Node>();
                foreach (Node node in GameInterface.world.Nodes)
                {
                    if (node.Owner == this)
                    {
                        owned.Add(node);
                    }
                }
                return owned;
            }
        }

        public Player(Color setColor)
        {
            color = setColor;
            home = new NodeHome();
            home.Owner = this;
        }

        public void UseGrowth()
        {
            foreach (Node node in Owned)
            {
                if (node is NodeGrow && !node.Used)
                {
                    node.Used = true;
                    break;
                }
            }
        }

        public void UseMove()
        {
            foreach (Node node in Owned)
            {
                if (node is NodeMove && !node.Used)
                {
                    node.Used = true;
                    break;
                }
            }
        }

        public void ShowSupplies()
        {
            List<NodeSupply> supplyNodes = new List<NodeSupply>();
            foreach (Node node in Owned)
            {
                if (node is NodeSupply)
                {
                    supplyNodes.Add(node as NodeSupply);
                }
            }
            ShowSupplies(supplyNodes);
        }

        void ShowSupplies(List<NodeSupply> supplyNodes)
        {
            int supplied = Owned.Count - 1;

            if (supplied >= 5)
            {
                home.freeSupplies = 0;
                supplied -= 5;
            }
            else if (supplied > 0)
            {
                home.freeSupplies = 5 - supplied;
                supplied = 0;
            }
            else
            {
                home.freeSupplies = 5;
            }

            foreach (NodeSupply supply in supplyNodes)
            {
                if (supplied >= 4)
                {
                    supply.freeSupplies = 0;
                    supplied -= 4;
                }
                else if (supplied > 0)
                {
                    supply.freeSupplies = 4 - supplied;
                    supplied = 0;
                }
                else
                {
                    supply.freeSupplies = 4;
                }
            }
        }
    }
}
