﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GrowthControl
{
    class Spit
    {
        Sprite graphics;
        Vector2 position;
        Vector2 velocity;
        NodeAttack attacker;
        Player owner;
        Color color;
        List<Particle> particles;
        bool collided;

        public Spit(NodeAttack setAttacker, Vector2 setVelocity)
        {
            position = setAttacker.Position;
            velocity = setVelocity;
            owner = setAttacker.Owner;
            attacker = setAttacker;
            graphics = new Sprite(GameServices.content);
            graphics.ChangeContent("Symbols");
            graphics.Source = new Rectangle(210, 60, 20, 20);
            color = new Color(owner.PlayerColor.R + 100, owner.PlayerColor.G + 100, owner.PlayerColor.B + 100);

            particles = new List<Particle>();
            Update();
        }

        public void Update()
        {
            if (!collided)
            {
                position += velocity * (float)GameServices.time.ElapsedGameTime.TotalSeconds;
                graphics.Position = position;
                CheckCollisions();
                Random rnd = new Random((int)(position.X * position.Y));
                for (int i = 0; i < 2; i++)
                {
                    particles.Add(new Particle(position, velocity * 0.5f + new Vector2(-100 + rnd.Next(200), -100 + rnd.Next(200)), color, this));
                }
            }
            foreach (Particle particle in particles)
            {
                particle.Update();
            }
        }

        void CheckCollisions()
        {
            foreach (Connection connection in GameInterface.world.Connections.ToArray())
            {
                //Checking closest distance from point to line:
                float t = Vector2.Dot(position - connection.Node1.Position, connection.Node2.Position - connection.Node1.Position) / Vector2.Dot(connection.Node2.Position - connection.Node1.Position, connection.Node2.Position - connection.Node1.Position);
                if (t < 0) t = 0;
                if (t > 1) t = 1;
                Vector2 point = connection.Node1.Position + t * (connection.Node2.Position - connection.Node1.Position);
                if (owner != connection.Owner && Vector2.Distance(point, position) < 10)
                {
                    connection.Break();
                }
            }

            foreach (Node node in GameInterface.world.Nodes)
            {
                if (Vector2.Distance(position, node.Position) < node.Size + 10 && node != attacker)
                {
                    collided = true;
                    Random rnd = new Random((int)(position.X * position.Y));
                    for (int i = 0; i < 50; i++)
                    {
                        Vector2 pVelocity = new Vector2(-500 + rnd.Next(1000), -500 + rnd.Next(1000));
                        pVelocity.Normalize();
                        particles.Add(new Particle(position, pVelocity * rnd.Next(1000), color, this));
                    }
                }

                if (position.X > 1950 || position.X < -30 || position.Y < -30 || position.Y > 1110)
                {
                    collided = true;
                }
            }
        }

        public void RemoveParticle(Particle particle)
        {
            particles.Remove(particle);
            if (particles.Count == 0)
            {
                GameInterface.world.Spits.Remove(this);
            }
        }

        public void Draw()
        {
            if (!collided)
            {
                graphics.Draw(color);
            }
            foreach (Particle particle in particles.ToArray())
            {
                particle.Draw();
            }
        }
    }
}
