﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace GrowthControl
{
    class Sprite
    {
        Vector2 position;
        Rectangle size;
        float rotation;
        float scale;

        string assetName;
        Vector2 origin;
        Rectangle source;
        Texture2D texture;
        Color tint;

        ContentManager content;

        public Sprite(ContentManager contentManager)
        {
            content = contentManager;
            position = new Vector2(0, 0);
            scale = 1.0f;
            tint = Color.White;
        }

        public Vector2 Position
        {
            get { return position; }
            set
            {
                position = value;
                size = new Rectangle((int)(position.X - source.Width * scale / 2), (int)(position.Y - source.Height * scale / 2), (int)(source.Width * scale), (int)(source.Height * scale));
            }
        }

        public Rectangle Size
        {
            get { return size; }
        }

        public float Rotation
        {
            get { return rotation; }
            set { rotation = value; }
        }

        public float Scale
        {
            get { return scale; }
            set
            {
                scale = value;
                size = new Rectangle((int)(position.X - source.Width * scale / 2), (int)(position.Y - source.Height * scale / 2), (int)(source.Width * scale), (int)(source.Height * scale));
            }
        }

        public Rectangle Source
        {
            get { return source; }
            set
            {
                source = value;
                size = new Rectangle((int)(position.X - source.Width * scale / 2), (int)(position.Y - source.Height * scale / 2), (int)(source.Width * scale), (int)(source.Height * scale));
                origin = new Vector2(source.Width / 2f, source.Height / 2f);
            }
        }

        public Vector2 Origin
        {
            get { return origin; }
            set
            {
                origin = value;
            }
        }

        public Texture2D Texture
        {
            get
            {
                return texture;
            }
        }

        public void ChangeContent(string assetname)
        {
            this.assetName = assetname;
            texture = content.Load<Texture2D>(assetName);
            Source = new Rectangle(0, 0, texture.Width, texture.Height);
        }

        public void Draw()
        {
            GameServices.spriteBatch.Draw(texture, position, source, tint, rotation, origin, scale, SpriteEffects.None, 0);
        }

        public void Draw(Color tint)
        {
            GameServices.spriteBatch.Draw(texture, position, source, tint, rotation, origin, scale, SpriteEffects.None, 0);
        }

        public void Draw(Vector2 customScale)
        {
            GameServices.spriteBatch.Draw(texture, position, source, tint, rotation, origin, customScale, SpriteEffects.None, 0);
        }

        public void Draw(Vector2 customScale, Color tint)
        {
            GameServices.spriteBatch.Draw(texture, position, source, tint, rotation, origin, customScale, SpriteEffects.None, 0);
        }

        public Color GetPixelColor(Vector2 position)
        {
            return GetPixelColor((int)position.X, (int)position.Y);
        }

        public Color GetPixelColor(int x, int y)
        {
            Color[] textureData = new Color[source.Height * source.Width];
            texture.GetData<Color>(0, source, textureData, 0, source.Width * source.Height);
            //texture.GetData<Color>(textureData);
            return textureData[(x - size.X) + (y - size.Y) * source.Width];
        }
    }
}
